@@ -125,6 +125,7 @@ bool path_exists(const char *path, struct stat *statbuf, bool follow_symlink);
 unsigned long path_mount_flags(const char *path);
 void path_split(const char *path, char **dir, char **base);
 bool path_subdir_p(const char *base, const char *path);
-char *realpath_safe(const char *path);
+char *realpath_(const char *path, bool fail_ok);
+void replace_char(char *str, char old, char new);
 void split(char **a, char **b, const char *str, char del);
 void version(void);
